/***************************************************************************
 *   Copyright (C) 2005 by Oliver Saal                                     *
 *   http://www.oliver-saal.de/                                            *
 *   osaal@gmx.de                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#ifndef DATA_H
#define DATA_H

#include <qstring.h>
#include <qptrlist.h>
#include <qpixmap.h>

double GetCostsPerKm (const unsigned int uPollutant, const unsigned int uAxes);

class CNode;
class CLink;
class CNetwork;
class CRoute;

class CCanvasNode;
class CCanvasLink;

class CNode
{
public:
	CNode ();
	~CNode ();

	enum status {StatusOff, StatusOn, StatusUnderConstruction};
	enum typ {TypUnknown, TypAD, TypAK, TypAS, TypAN, TypBS, TypBG, TypTR};

	static typ GetTyp (const QString strShortcut);
	static status GetStatus (const char c);

	QString GetStatusText () const;
	QString GetTypeShortcut () const;
	QString GetTypeText () const;

	inline bool IsCross ()				{ return (m_typ == TypAD || m_typ == TypAK); }
	bool IsOneWay ();
	void debug (void) const;

	QString GetUniqueString (void) const;
	QString AsHTML (void);
	QPixmap GetPixmap16 (void) const;

//	bool FindRoute (CRoute *pRoute);

	QString WarningText (void);


	CNode *FindNextSubNodeToDisplay (void);
	CNode *FindNextLeftNodeToDisplay (void);	// one way outgoing
	CNode *FindNextRightNodeToDisplay (void);	// one way incoming

public:
	QString m_strName;
	int m_iBAB;
	QString m_strNumber;

	status m_status;
	typ m_typ;

	QPtrList<CLink> m_listLinkOut;
	QPtrList<CLink> m_listLinkIn;

	CLink *m_pNearestLink;
	float m_fNearestKm;

	unsigned m_uCSVLine;

	CCanvasNode *m_pCanvasNode;
};

bool operator == (CNode& n1, CNode& n2);

class CLink
{
public:
	CLink ();
	CLink (CNode *pNodeFrom, CNode *pNodeTo, const float fKm, const unsigned uCSVLine=0);
	~CLink ();

public:
	CNode *m_pNodeFrom;
	CNode *m_pNodeTo;

	float m_fKm;

	unsigned m_uCSVLine;

	CCanvasLink *m_pCanvasLink;
};

class CNetwork
{
public:
	CNetwork ();
	~CNetwork ();

	bool Load (const QString strFile, QWidget *pParent, const bool bForce=false);

	void CalculateNearestDistances (CNode *pNodeFrom);
	void CalculateRoute (CNode *pNodeFrom, CNode *pNodeTo, QPtrList<CLink> &listLink);


	void CreateSampleNetwork (void);

	void FindNode (const QString strSearch, QPtrList<CNode> &listNode, const bool bAllowCrosses=true);
	CNode* FindNextNodeToDisplay (const int iBAB, CNode *pOldNode=NULL);

	QString Check (void);

protected:
	CNode *AddNode (CNode &node);
	void CreateCrossLinks (void);

	QString m_strDBFile;

public:
	QPtrList<CNode> m_listNode;
	QPtrList<CLink> m_listLink;

	CNode *m_pCalculatedFrom;
};

class CRoute 
{
public:
	CRoute (CNode *pNodeFrom, CNode *pNodeTo);
	~CRoute ();

	bool Find (void);
	float Distance (void);

	unsigned int CountLinks (const CLink *pLink);

public:
	CNode *m_pNodeFrom;
	CNode *m_pNodeTo;
	QPtrList<CLink> m_listLink;
};


#endif

