/***************************************************************************
 *   Copyright (C) 2005 by Oliver Saal                                     *
 *   http://www.oliver-saal.de/                                            *
 *   osaal@gmx.de                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include "dlgbrowser2.h"

#include <qpushbutton.h>
#include <qheader.h>
#include <qlistview.h>
#include <qtextbrowser.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qsplitter.h>
#include <qpainter.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qimage.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qpoint.h>
#include <qrect.h>

#include <math.h>

#include "data.h"
#include "icon.h"
#include "vector2d.h"

static const char* const image0_data[] = { 
"16 16 77 2",
"Qt c None",
"#h c #004000",
"#e c #004200",
"#k c #004300",
"#j c #004600",
"#i c #004700",
"#f c #004800",
"#b c #004900",
".9 c #004d00",
".8 c #005100",
".2 c #005300",
".1 c #005500",
".0 c #005c00",
".T c #005f00",
".D c #006000",
".S c #006200",
".C c #006600",
".E c #006700",
".K c #006800",
".z c #006b00",
"#. c #006c00",
"#a c #006e00",
".t c #006f00",
".s c #007000",
".u c #007300",
".J c #007400",
"#g c #007800",
".k c #007900",
".j c #007a00",
".N c #007c00",
".f c #008000",
".l c #008100",
".o c #008300",
".F c #008600",
".e c #008800",
".p c #008900",
".d c #008c00",
".# c #008d00",
".g c #008e00",
".a c #009200",
".Z c #009300",
".3 c #009c00",
"#d c #00ac00",
".R c #00ae00",
".O c #00b100",
"#c c #00b200",
"## c #00be00",
".U c #00ca00",
".7 c #00cf00",
".W c #00d300",
".X c #00d500",
".5 c #00da00",
".4 c #00db00",
".M c #00dd00",
".6 c #00e000",
".P c #00e900",
".Y c #00ee00",
".V c #00f100",
".L c #00f300",
".Q c #00fe00",
".G c #01c001",
".y c #038203",
".h c #038903",
".v c #049104",
".i c #058f05",
".b c #069800",
".A c #06b106",
".r c #089908",
".B c #08ed08",
".c c #099b00",
".H c #0af50a",
".I c #0bbe0b",
".m c #11a111",
".q c #18b918",
".n c #19a719",
".w c #1bce1b",
".x c #1fbd1f",
"QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
"QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
".#QtQtQtQtQtQtQtQtQtQtQtQt.a.b.c",
".d.eQtQtQtQtQtQtQtQtQt.f.d.#.g.a",
".h.i.jQtQtQtQtQtQt.k.l.m.n.o.pQt",
".j.q.r.sQtQtQt.t.u.v.w.x.y.fQtQt",
".z.A.B.e.C.D.E.F.G.H.I.J.sQtQtQt",
".K.o.L.M.N.u.O.P.Q.R.S.CQtQtQtQt",
"Qt.T.U.V.W.X.Y.P.Z.0.1QtQtQtQtQt",
"Qt.2.3.4.5.6.7.o.8QtQtQtQtQtQtQt",
"Qt.9#..U.7###a#bQtQtQtQtQtQtQtQt",
"QtQt.8#c#d.D#eQtQtQtQtQtQtQtQtQt",
"QtQt#f#g.T#hQtQtQtQtQtQtQtQtQtQt",
"QtQt#i#j#kQtQtQtQtQtQtQtQtQtQtQt",
"QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
"QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"};

static const char* const image1_data[] = { 
"16 16 32 1",
"# c None",
"z c #880400",
"q c #931a00",
"B c #990100",
"b c #9e2c00",
"g c #ad2700",
"v c #af0b00",
"r c #b40d00",
"t c #c90400",
"n c #cb1000",
"c c #d11d02",
"y c #d30b01",
"h c #d42604",
"d c #dd350b",
"i c #e54a14",
"a c #e84300",
"f c #eb591b",
"D c #ec0000",
"e c #f2863c",
"A c #f61802",
"l c #f66528",
"x c #f8340a",
"u c #f8551d",
"C c #f90200",
"p c #f99e60",
"s c #fb9153",
"k c #fbac73",
"m c #fd7638",
"w c #fe4d17",
"o c #feba89",
"j c #fec9a0",
". c #ffffff",
"................",
".##ab#######c###",
".#defg#####hic##",
".aejkig###hlmin#",
".bfkopiq#hlmldr#",
".#gisksdclmldt##",
".##gissmlmldr###",
".###qdmsmldr####",
".####clmlucv####",
".###hlmluwxyz###",
".##hlmldcxxAyB##",
".#hlmldrvyACCtB#",
".cimldr##zyCCDt#",
".#cfdt####BtDtB#",
".##nr######BtB##",
".###############"};

static const char* const image2_data[] = { 
"16 16 100 2",
".g c None",
"#o c #727297",
".f c #76769b",
".G c #77779a",
".s c #78789b",
".y c #79799d",
".m c #7a7a9d",
"#n c #7c7c9e",
".O c #8080a0",
"#H c #8080a2",
".r c #8282a3",
".8 c #8686a8",
".X c #8787a8",
"#b c #8787a9",
"#m c #8b8ba9",
"#s c #8b8baa",
"#B c #8c8cab",
".q c #9292b1",
"#F c #9696b3",
"#G c #9797b3",
"#E c #9797b6",
"#D c #9a9ab6",
"#C c #9b9bb7",
".W c #9e9eb9",
"#p c #a0a0bc",
".e c #a6a6c2",
".x c #ababc5",
".N c #ababc6",
".p c #b2b2cc",
".D c #b3b3ca",
".F c #b6b6cf",
".4 c #b7b7ce",
"#t c #b8b8cf",
".C c #b9b9d0",
".B c #bfbfd6",
"#w c #c0c0d7",
"#z c #c2c2d7",
"#c c #c3c3d7",
".E c #c4c4d8",
"Qt c #c5c5da",
".J c #c6c6da",
"#h c #c6c6dc",
".w c #c8c8dc",
".S c #c9c9dd",
".0 c #cacade",
"## c #cccce0",
".h c #cecede",
".7 c #cecee1",
".c c #cfcfe1",
".9 c #d0d0e0",
".b c #d1d1e2",
".d c #d1d1e4",
".A c #d2d2e6",
".a c #d5d5e4",
".# c #d6d6e6",
".l c #d7d7e7",
"#k c #d7d7e9",
"#l c #dadaec",
".V c #dcdcec",
"#j c #dcdced",
"#a c #ddddec",
"#g c #dddded",
"#r c #dedeee",
"#f c #e0e0ee",
".v c #e0e0f0",
".M c #e1e1f0",
".I c #e2e2f0",
".3 c #e2e2f1",
"#v c #e2e2f3",
"#e c #e4e4f1",
".R c #e4e4f3",
".L c #e6e6f3",
"#u c #e7e7f5",
"#q c #e8e8f5",
"#A c #e8e8f7",
".Q c #e9e9f6",
".Z c #ececf7",
".k c #ededf7",
".Y c #ededf8",
".6 c #eeeef8",
".u c #f0f0f9",
"#. c #f0f0fc",
".K c #f1f1f8",
"#d c #f1f1f9",
".U c #f1f1fc",
".z c #f3f3fc",
".o c #f3f3fd",
".H c #f3f3fe",
".2 c #f5f5fd",
".P c #f6f6fe",
".1 c #f7f7fc",
".5 c #f7f7fe",
".T c #f8f8fc",
"#i c #f8f8fd",
"#y c #f8f8fe",
"#x c #f9f9fd",
".j c #f9f9fe",
".n c #fcfcfe",
".t c #fdfdfe",
".i c #fefefe",
"Qt.#.a.b.c.d.e.f.g.g.g.g.g.g.g.g",
".h.i.i.i.j.k.l.e.m.g.g.g.g.g.g.g",
".h.i.i.i.n.o.p.q.r.s.g.g.g.g.g.g",
".h.i.i.i.t.u.v.w.x.y.g.g.g.g.g.g",
".h.i.i.i.i.z.A.B.C.D.w.E.F.G.g.g",
".h.i.i.i.H.I.J.K.i.i.H.L.M.N.O.g",
".h.i.i.P.Q.R.S.T.i.i.i.U.V.W.X.s",
".h.i.t.u.Y.Z.0.1.i.i.i.2.3.4.N.r",
".h.i.5.U.6.Z.0.1.i.i.i.5.Q.7.J.8",
".9.i.5.2.z#.##.1.i.i.i.Y#a##.c#b",
"#c#d#e.I#f#g#h#i.i.i.U.3#j#k#l#m",
"#n#n#n#n#n#o#p.i.i.P.k#q.R#r.v#s",
".g.g.g.g.g.g#t.n.i.z.u.Q#u.M#v#s",
".g.g.g.g.g.g#w#x#y.o.u.Z#u.M#v#s",
".g.g.g.g.g.g#z.j.t.j.5.U.Y#u#A#B",
".g.g.g.g.g.g#B#C#C#C#D#E#E#F#G#H"};

class CBrowserItem2 : public QListViewItem
{
public:
	CBrowserItem2 (QListView *pParent, const int iBAB) :
		QListViewItem (pParent, QString ("A %1").arg (iBAB))
	{
		m_pNode = NULL;
		m_iBAB = iBAB;
	}

	CBrowserItem2 (QListViewItem *pParent, CNode *pNode) :
		QListViewItem (pParent, QString("A %1").arg(pNode->m_iBAB), pNode->m_strNumber, pNode->m_strName)
	{
		setPixmap(1, pNode->GetPixmap16());
		m_pNode = pNode;
		m_iBAB = pNode->m_iBAB;

		if (!pNode->WarningText().isEmpty())
			setPixmap (2, IconWarning16());
	}

	CNode *m_pNode;
	int m_iBAB;
};

class CCanvasNode : public QCanvasRectangle
{
public:
	CCanvasNode (CNode *pNode, QCanvas * canvas);
	~CCanvasNode ();

	inline CNode *node () { return m_pNode; }

protected:
	virtual void drawShape (QPainter &painter);

protected:
	CNode *m_pNode;
};

CCanvasNode::CCanvasNode (CNode *pNode, QCanvas * canvas) : QCanvasRectangle (canvas)
{
	m_pNode = pNode;
	setPen (Qt::black);
	setBrush (Qt::white);
	setSize (200, 30);
	m_pNode->m_pCanvasNode = this;
	setZ(1);
}

CCanvasNode::~CCanvasNode ()
{
	hide ();
	m_pNode->m_pCanvasNode = NULL;
}

void CCanvasNode::drawShape (QPainter &painter)
{
QString str;
QRect r = rect ();
QPixmap pix = m_pNode->GetPixmap16 ();

	r.setLeft (r.left() + 25);
	r.setRight (r.right() - 5);
	QCanvasRectangle::drawShape (painter);
	if (m_pNode->m_strNumber.isEmpty())
		str = QString ("A %1 %2").arg (m_pNode->m_iBAB).arg (m_pNode->m_strName);
	else
		str = m_pNode->GetUniqueString ();
	painter.drawText (r, AlignLeft | AlignVCenter, str);

	if (!pix.isNull())
	{
		painter.drawPixmap (QPoint (rect().left() + 4, rect().center().y() - pix.height()/2), pix);
	}
}


class CCanvasLink : public QCanvasLine //QCanvasItem
{
public:
	CCanvasLink (CLink *pLink, QCanvas *canvas);
	~CCanvasLink ();

	void calculatePoints ();
/*
	QPoint startPoint () const		{ return m_ptFrom; }
	QPoint endPoint () const		{ return m_ptTo; }

	virtual bool collidesWith (const QCanvasItem *other) const;
    virtual bool collidesWith( const QCanvasSprite*,
			       const QCanvasPolygonalItem*,
			       const QCanvasRectangle*,
			       const QCanvasEllipse*,
			       const QCanvasText* ) const
								{ return false; }
*/
protected:
	virtual void drawShape (QPainter &painter);

//	virtual void draw (QPainter &painter) { drawShape (painter); }
	virtual QRect boundingRect () const;

protected:
	CLink *m_pLink;

//	QPoint m_ptFrom;
//	QPoint m_ptTo;
};

void CCanvasLink::calculatePoints ()
{
QRect rectFrom, rectTo;
QPoint ptFrom, ptTo;
double dToOffset=0.25, dFromOffset=0.25;

	if (!m_pLink || !m_pLink->m_pNodeFrom || !m_pLink->m_pNodeTo || !m_pLink->m_pNodeFrom->m_pCanvasNode || !m_pLink->m_pNodeTo->m_pCanvasNode) return;

	rectFrom = m_pLink->m_pNodeFrom->m_pCanvasNode->rect();
	rectTo = m_pLink->m_pNodeTo->m_pCanvasNode->rect();

	//if (m_pLink->m_pNodeFrom->m_listLinkOut.count() == 1) dFromOffset = 0.0;
	//if (m_pLink->m_pNodeTo->m_listLinkIn.count() == 1) dToOffset = 0.0;
	if (m_pLink->m_pNodeFrom->IsOneWay()) dFromOffset = 0.0;
	if (m_pLink->m_pNodeTo->IsOneWay()) dToOffset = 0.0;

	if (rectFrom.right() < rectTo.left())
	{ // rectFrom is right
		ptFrom = QPoint (rectFrom.right(), rectFrom.center().y() - rectFrom.height()*dFromOffset);
		ptTo = QPoint (rectTo.left(), rectTo.center().y() - rectTo.height()*dToOffset);
	}
	else if (rectTo.right() < rectFrom.left())
	{ // rectFrom is left
		ptFrom = QPoint (rectFrom.left(), rectFrom.center().y() + rectFrom.height()*dFromOffset);
		ptTo = QPoint (rectTo.right(), rectTo.center().y() + rectTo.height()*dToOffset);
	}
	else if (rectFrom.bottom () < rectTo.top ())
	{ // rectFrom is top
		ptFrom = QPoint (rectFrom.center().x() - rectFrom.width()*dFromOffset, rectFrom.bottom());
		ptTo = QPoint (rectTo.center().x() - rectTo.width()*dToOffset, rectTo.top());
	}
	else if (rectTo.bottom () < rectFrom.top ())
	{ // rectFrom is bottom
		ptFrom = QPoint (rectFrom.center().x() + rectFrom.width()*dFromOffset, rectFrom.top());
		ptTo = QPoint (rectTo.center().x() + rectTo.width()*dToOffset, rectTo.bottom());
	}
	setPoints (ptFrom.x(), ptFrom.y(), ptTo.x(), ptTo.y());
}

CCanvasLink::CCanvasLink (CLink *pLink, QCanvas *canvas) : 
	QCanvasLine /*QCanvasItem*/ (canvas)
{
	m_pLink = pLink;
	m_pLink->m_pCanvasLink = this;
	calculatePoints ();
	setZ(0);
}

CCanvasLink::~CCanvasLink ()
{
	hide ();
	m_pLink->m_pCanvasLink = NULL;
}
/*
bool CCanvasLink::collidesWith (const QCanvasItem *other) const
{
	return false;
}
*/

QRect unite (const QRect &rect, const QPoint pt)
{
QRect ret = rect;

	if (ret.isNull())
	{
		ret.setTopLeft (pt);
		ret.setBottomRight (pt);
		return ret;
	}
	if (pt.x() < ret.left())
		ret.setLeft (pt.x());
	if (pt.x() > ret.right ())
		ret.setRight (pt.x());

	if (pt.y() < ret.top ())
		ret.setTop(pt.y());
	if (pt.y() > ret.bottom())
		ret.setBottom (pt.y());
	return ret;
}

QRect CCanvasLink::boundingRect () const
{
QRect rect = QCanvasLine::boundingRect();
//	rect = unite (rect, startPoint());
//	rect = unite (rect, endPoint ());

	if (m_pLink && m_pLink->m_fKm > 0 && rect.width() < 80)
	{
		rect.setLeft (rect.center().x() - 40);
		rect.setRight (rect.center().x() + 40);
	}
	if (rect.width() < 20)
	{
		rect.setLeft (rect.center().x() - 10);
		rect.setRight (rect.center().x() + 10);
	}
	if (rect.height() < 20)
	{
		rect.setTop (rect.center().y() - 10);
		rect.setBottom (rect.center().y() + 10);
	}

	return (rect);
}

void CCanvasLink::drawShape (QPainter &painter)
{
QPoint ptStart=startPoint(), ptEnd=endPoint(); 
QRect rect (ptStart, ptEnd);
double dLen=20, m=0, dAlpha=0;
CVector2D vTip, vDir, vHelp, vArrow1, vArrow2;

	if (ptStart.x() == ptEnd.x())
	{
		rect.moveBy (-100, 0);
		rect.setWidth (200);
	}

//	painter.fillRect (boundingRect(), gray);
	QCanvasLine::drawShape (painter);

	if (m_pLink->m_fKm > 0) 
		painter.drawText (rect, AlignCenter, QString ("%1 km").arg ((double)m_pLink->m_fKm, 0, 'f', 1));

	vTip = startPoint();
	vDir = CVector2D (endPoint()) - CVector2D (startPoint());
	vDir.makeIdentityLen ();
	if (vDir.isNull()) return;
	vHelp = ptEnd - 10 * vDir;
	vArrow1 = vHelp + 3 * vDir.vert();
	vArrow2 = vHelp - 3 * vDir.vert();

	painter.drawLine (ptEnd, vArrow1.toPoint());
	painter.drawLine (ptEnd, vArrow2.toPoint());
}


CDlgBrowser2::CDlgBrowser2 (CNetwork *pNetwork, QWidget *pParent, const char *name) : QDialog (pParent, name, true, WStyle_Maximize | WStyle_SysMenu)
{
QHBoxLayout* layout3=NULL, *hlayButtons=NULL;
QSpacerItem* spacer=NULL;
QImage image0 (image0_data);
QImage image1 (image1_data);
QImage image2 (image2_data);
#include "filesave.xpm"
QImage imageSave (filesave);

	m_pNetwork = pNetwork;
	m_iBABGraph = 0;

    if (!name) setName( "CDlgBrowser2" );
    m_mainlayout = new QVBoxLayout( this, 11, 6, "CDlgBrowser2Layout"); 

	m_splitterHor = new QSplitter (Qt::Horizontal, this);
	m_listView = new QListView (m_splitterHor);
    m_listView->setMinimumSize( QSize( 330, 400 ) );
    m_listView->setMaximumSize( QSize( 400, 32767 ) );
	m_listView->setSortColumn(-1);
	m_listView->setRootIsDecorated(true);
	m_listView->setAllColumnsShowFocus(true);

	m_splitterVert = new QSplitter (Qt::Vertical, m_splitterHor);

	//vlayGraph = new QVBoxLayout (m_splitterVert, 0, 6, "vlayGraph");

	m_canvasView = new QCanvasView (&m_canvas, m_splitterVert);
	m_canvasView->setMinimumSize (QSize (500, 300));
	//vlayGraph->addWidget (m_canvasView);


	m_text = new QTextBrowser (m_splitterVert);
	m_text->setMinimumSize (QSize (500, 300));

	m_mainlayout->addWidget (m_splitterHor);

	hlayButtons = new QHBoxLayout (this, 0, 6, "hlayButtons");
    spacer = new QSpacerItem( 40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	hlayButtons->addItem (spacer);
	m_pbGraphCopy = new QPushButton (this, "GraphCopy");
	m_pbGraphCopy->setIconSet (QIconSet (image2));
	hlayButtons->addWidget (m_pbGraphCopy);
	m_pbGraphSave = new QPushButton (this, "GraphSave");
	m_pbGraphSave->setIconSet (QIconSet (imageSave));
	hlayButtons->addWidget (m_pbGraphSave);
	hlayButtons->addItem (new QSpacerItem (10, 20));
	m_pbTextCopy = new QPushButton (this, "TextCopy");
	m_pbTextCopy->setIconSet (QIconSet (image2));
	hlayButtons->addWidget (m_pbTextCopy);
	m_pbTextSave = new QPushButton (this, "TextSave");
	m_pbTextSave->setIconSet (QIconSet (imageSave));
	hlayButtons->addWidget (m_pbTextSave);
	m_mainlayout->addLayout (hlayButtons);

	layout3 = new QHBoxLayout( 0, 0, 6, "layout3"); 
    spacer = new QSpacerItem( 40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    layout3->addItem( spacer );

    m_pbOK = new QPushButton( this, "m_pbOK" );
    m_pbOK->setIconSet( QIconSet( image0 ) );
    layout3->addWidget( m_pbOK );

	m_pbCancel = new QPushButton( this, "m_pbCancel" );
	m_pbCancel->setIconSet( QIconSet( image1 ) );
	layout3->addWidget( m_pbCancel );
	QSpacerItem* spacer_2 = new QSpacerItem( 40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout3->addItem( spacer_2 );
	m_mainlayout->addLayout( layout3 );
	languageChange();
	resize( QSize(400, 400).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );

	// signals and slots connections
	connect( m_listView, SIGNAL( selectionChanged(QListViewItem*) ), this, SLOT( onItemChanged(QListViewItem*) ) );
	connect( m_listView, SIGNAL( expanded(QListViewItem*) ), this, SLOT( onItemExpanded(QListViewItem*) ) );
	connect( m_pbOK, SIGNAL( clicked() ), this, SLOT( onOK() ) );
	connect( m_pbCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( m_pbGraphCopy, SIGNAL( clicked() ), this, SLOT( onGraphCopy() ) );
	connect( m_pbGraphSave, SIGNAL( clicked() ), this, SLOT( onGraphSave() ) );
	connect( m_pbTextCopy, SIGNAL( clicked() ), this, SLOT( onTextCopy() ) );
	connect( m_pbTextSave, SIGNAL( clicked() ), this, SLOT( onTextSave() ) );

	m_pbGraphCopy->setEnabled (false);
	m_pbGraphSave->setEnabled (false);
	m_pbTextCopy->setEnabled (false);
	m_pbTextSave->setEnabled (false);

#ifdef Q_OS_WIN32
	m_pbGraphCopy->hide ();
	m_pbTextCopy->hide ();
#endif

	initTree ();
}

CDlgBrowser2::~CDlgBrowser2 ()
{
}

void CDlgBrowser2::languageChange()
{
    setCaption( tr( "TollCalculator - Browser" ) );
    m_pbOK->setText( tr( "&OK" ) );
    m_pbOK->setAccel( QKeySequence( tr( "Alt+O" ) ) );
    m_pbCancel->setText( tr( "&Cancel" ) );
    m_pbCancel->setAccel( QKeySequence( tr( "Alt+C" ) ) );

	if (m_listView->columns() == 0)
	{
		m_listView->addColumn (tr("BAB"), 80);
	    m_listView->addColumn (tr("Node"), 50);
	    m_listView->addColumn (tr("Name"),170);
	}
	else
	{
		m_listView->setColumnText (0, tr("BAB"));
	    m_listView->setColumnText (1, tr("Node"));
	    m_listView->setColumnText (2, tr("Name"));
	}

	m_pbGraphCopy->setText (tr("Copy graph"));
	m_pbGraphSave->setText (tr("Save graph..."));
	m_pbTextCopy->setText (tr("Copy text"));
	m_pbTextSave->setText (tr("Save text..."));
}

void CDlgBrowser2::initTree ()
{
int iBAB=0;
CNode *pNode = NULL;
CBrowserItem2 *pBABItem=NULL, *pItem=NULL;
    m_pNetwork->m_listNode.last();
    while ((pNode = m_pNetwork->m_listNode.current()) != NULL)
    {
		if (iBAB != pNode->m_iBAB || pBABItem == NULL)
		{
		    pBABItem = new CBrowserItem2 (m_listView, pNode->m_iBAB);
			pBABItem->setExpandable(true);
			iBAB = pNode->m_iBAB;
		}
		//pItem = new CBrowserItem2 (pBABItem, pNode);
		m_pNetwork->m_listNode.prev();
    }
}

void CDlgBrowser2::initTree (CBrowserItem2 *pBABItem)
{
CNode *pNode = NULL;
	if (pBABItem->m_pNode != NULL) return;
	if (pBABItem->childCount() > 0) return;
	for (pNode = m_pNetwork->m_listNode.last(); pNode; pNode = m_pNetwork->m_listNode.prev())
	{
		if (pNode->m_iBAB == pBABItem->m_iBAB)
			new CBrowserItem2 (pBABItem, pNode);
	}
}

void CDlgBrowser2::onItemExpanded (QListViewItem *pItem)
{
CBrowserItem2 *pItem2=(CBrowserItem2*)pItem;
	initTree (pItem2);
}

void CDlgBrowser2::onItemChanged( QListViewItem *pItem )
{
CBrowserItem2 *pItem2=(CBrowserItem2*)pItem;
QString str;
int iBAB=0;


	if (pItem2->m_pNode != NULL)
	{
		str = pItem2->m_pNode->AsHTML();
	}
	m_text->setText(str);
	updateGraph (pItem2->m_iBAB);
	initTree (pItem2);

	bool bEnable = !str.isEmpty();
	m_pbTextCopy->setEnabled (bEnable);
	m_pbTextSave->setEnabled (bEnable);
}

void CDlgBrowser2::clearGraph ()
{
QCanvasItemList list = m_canvas.allItems();
QCanvasItemList::Iterator it = list.begin();
	for (; it != list.end(); ++it)
	{
		if ( *it ) delete *it;
    } 

//	m_pbGraphCopy->setEnabled (false);
//	m_pbGraphSave->setEnabled (false);
}


void CDlgBrowser2::updateGraph (const int iBAB)
{
CNode *pMainNode=NULL, *pSubNode=NULL, *pLeftNode=NULL, *pRightNode=NULL;
CLink *pLink=NULL;
CCanvasNode *cnMain=NULL, *cnSub=NULL, *cnLeft=NULL, *cnRight=NULL;
CCanvasLink *cl=NULL;
//int y=30, y2=0;
int yMain=30, yLeft=0, yRight=0, ySub=0, xRight=0, xLeft=999, xMove=0;

	if (iBAB == m_iBABGraph) return;
	clearGraph();


	while ((pMainNode = m_pNetwork->FindNextNodeToDisplay(iBAB, pMainNode)) != NULL)
	{
		cnMain = new CCanvasNode (pMainNode, &m_canvas);
		cnMain->move (200, yMain);
		cnMain->show ();
		if (cnMain->rect().right() > xRight) xRight = cnMain->rect().right();
		if (cnMain->rect().left() < xLeft) xLeft = cnMain->rect().left();

		ySub = yMain;
		yMain += cnMain->rect().height() + 30;
		cnSub = NULL;
		while ((pSubNode = pMainNode->FindNextSubNodeToDisplay ()) != NULL)
		{
			cnSub = new CCanvasNode (pSubNode, &m_canvas);
			cnSub->move (cnMain->rect().right() + 30, ySub);
			cnSub->show ();
			if (cnSub->rect().right() > xRight) xRight = cnSub->rect().right();
			if (cnSub->rect().left() < xLeft) xLeft = cnSub->rect().left();

			ySub += cnSub->height() + 30;
		}
		if (cnSub) cnMain->moveBy (0, (ySub-yMain)/2);
		if (ySub > yMain) yMain = ySub;

		yLeft = yMain;
		yRight = yMain;
		cnLeft = NULL;
		cnRight = NULL;

		while ((pLeftNode = pMainNode->FindNextLeftNodeToDisplay ()) != NULL)
		{
			cnLeft = new CCanvasNode (pLeftNode, &m_canvas);
			cnLeft->move (cnMain->rect().left() - cnLeft->rect().width()*3/4, yLeft);
			cnLeft->show ();
			if (cnLeft->rect().right() > xRight) xRight = cnLeft->rect().right();
			if (cnLeft->rect().left() < xLeft) xLeft = cnLeft->rect().left();

			yLeft += cnLeft->height() + 30;
		}

		while ((pRightNode = pMainNode->FindNextRightNodeToDisplay ()) != NULL)
		{
			cnRight = new CCanvasNode (pRightNode, &m_canvas);
			cnRight->move (cnMain->rect().right() - cnRight->rect().width()/4, yRight);
			cnRight->show ();
			if (cnRight->rect().right() > xRight) xRight = cnRight->rect().right();
			if (cnRight->rect().left() < xLeft) xLeft = cnRight->rect().left();

			yRight += cnRight->height() + 30;
		}
		if (yLeft > yMain) yMain = yLeft;
		if (yRight > yMain) yMain = yRight;
	}

	for (pLink = m_pNetwork->m_listLink.first(); pLink; pLink = m_pNetwork->m_listLink.next())
	{
		if (pLink->m_pNodeFrom->m_pCanvasNode == NULL || pLink->m_pNodeTo->m_pCanvasNode == NULL) continue;
		if (pLink->m_pNodeFrom->m_iBAB != iBAB && pLink->m_pNodeTo->m_iBAB != iBAB) continue;
		cl = new CCanvasLink (pLink, &m_canvas);
		cl->show ();
	}

/*	xMove = 0;
	if (xLeft > 30)
	{
		xMove = -(xLeft - 30);
		QCanvasItemList list = m_canvas.allItems();
		QCanvasItemList::Iterator it = list.begin();
		for (; it != list.end(); ++it)
		{
			(*it)->moveBy (xMove, 0);
		} 
	}
*/
	m_canvas.resize (xRight+30+xMove, yMain);
	m_canvas.update();
	m_iBABGraph = iBAB;

	m_pbGraphCopy->setEnabled (true);
	m_pbGraphSave->setEnabled (true);
}

void CDlgBrowser2::onTextSave ()
{
CBrowserItem2 *pItem2=(CBrowserItem2*)m_listView->selectedItem();
QString str;
int iBAB=0;
QFile file;

	if (pItem2 == NULL) return;
	if (pItem2->m_pNode == NULL) return;
	str = pItem2->m_pNode->AsHTML();

	QString strFile = QFileDialog::getSaveFileName (QString ("%1.html").arg (pItem2->m_pNode->GetUniqueString()), tr("HTML-File (*.html)"), this);
	if (strFile.isEmpty()) return;
	if (strFile.right (4) != "html") strFile += ".html";
	file.setName (strFile);
	file.open (IO_WriteOnly);
	QTextStream ts (&file);

	ts << "<html>\n<head></head>\n<body>\n";	
	ts << str;
	ts << "</body>\n";
}

void CDlgBrowser2::onTextCopy ()
{
	QApplication::clipboard()->setText (m_text->text(), QClipboard::Clipboard);
}

QRect getCanvasSize (QCanvas *pCanvas)
{
QCanvasItemList list = pCanvas->allItems();
QCanvasItemList::Iterator it = list.begin();
QCanvasItem *pItem;
QRect rect;

	for (it = list.begin(); it != list.end(); ++it)
	{
		pItem = *it;
		if (!pItem) continue;
		rect = rect.unite (pItem->boundingRect());
    } 
	return rect;
}

QImage getCanvasImage (QCanvas *pCanvas)
{
QRect rect = getCanvasSize (pCanvas);
QPixmap pix (rect.right(), rect.bottom());
QImage img;
QPainter painter (&pix);

	pCanvas->drawArea (rect, &painter);

	rect.setWidth (rect.width()-1);
	rect.setHeight (rect.height()-1);
	img = pix.convertToImage ().copy(rect).convertDepth (8);

	return (img);
}

void CDlgBrowser2::onGraphSave ()
{
QString strFile = QFileDialog::getSaveFileName (QString ("A %1.png").arg (m_iBABGraph), tr("Image (*.png)"), this);
	if (strFile.isEmpty()) return;
	if (strFile.right (3) != "png") strFile += ".png";
	getCanvasImage (&m_canvas).save (strFile, "PNG");
}

void CDlgBrowser2::onGraphCopy ()
{
	QApplication::clipboard()->setImage (getCanvasImage (&m_canvas), QClipboard::Clipboard);
}

void CDlgBrowser2::onOK ()
{
CBrowserItem2 *pItem2=(CBrowserItem2*)m_listView->selectedItem();
	if (pItem2 == NULL || pItem2->m_pNode == NULL)
	{
		QMessageBox::information (this, tr("Information"), tr("You have to select a node."));
		return;
	}
	accept();
}

CNode* CDlgBrowser2::goSelectNode ()
{
	if (exec () != Accepted) return NULL;

	CBrowserItem2 *pItem2=(CBrowserItem2*)m_listView->selectedItem();
	if (pItem2 == NULL) return NULL;
	return (pItem2->m_pNode);
}
