/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/



void CDlgTollCalculator::init()
{	
	m_pNodeFrom = NULL;
	m_pNodeTo = NULL;

	m_pAppTranslator = new QTranslator (this);
	m_pQtTranslator = new QTranslator (this);

	qApp->installTranslator (m_pAppTranslator);
	qApp->installTranslator (m_pQtTranslator);

	m_config.Load ();
	m_pAppTranslator->load("mautrechner_" + m_config.m_strLocale, m_config.m_strQMPath);
	m_pQtTranslator->load("qt_" + m_config.m_strLocale, m_config.m_strQMPath);

	m_lvRoute->setSortColumn(-1);
	m_lvRoute->setAllColumnsShowFocus(true);

	retranslateStrings();
	languageChange();
	updateCosts();

	m_checkAll->setChecked (m_config.m_bShowAll);
	m_cbPollutant->setCurrentItem (m_config.m_iPollutant);
	m_cbAxes->setCurrentItem (m_config.m_iAxes);

#ifdef Q_OS_WIN32
    m_pbCopy->hide ();
#endif

	m_pUpdater = new CUpdater (this);
	connect (m_pUpdater, SIGNAL (DatabaseReceived(QString)), this, SLOT(onDatabaseReceived(QString)));

	reloadDatabase (true);

	if (m_config.m_strDBFile.isEmpty())
	{
		if (QMessageBox::question (this, tr("Information"), 
			tr("There exists no toll tabular. Would you like to download the newest tabular now?\n\nNote: This program can not be used without the toll tabular."), QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
		{
			connect (m_pUpdater, SIGNAL (UpdateInfoReceived()), this, SLOT(onUpdateInfoReceived()));
			m_pUpdater->GetUpdateInfo (true);
		}
	}
	else if (m_config.UpdateCheckNow ())
	{
		connect (m_pUpdater, SIGNAL (UpdateInfoReceived()), this, SLOT(onUpdateInfoReceived()));
		m_pUpdater->GetUpdateInfo (true);
	}

	m_config.FillStartCombo (m_cbStart);
	m_config.FillDestinationCombo (m_cbDestination);
}


void CDlgTollCalculator::retranslateStrings()
{
    while (m_lvRoute->columns()) m_lvRoute->removeColumn(0);
    m_lvRoute->addColumn (tr("BAB"), 90);
    m_lvRoute->addColumn (tr("Node"), 50);
    m_lvRoute->addColumn(tr("Name"),150);
    m_lvRoute->addColumn(tr("Distance"), 70);
    m_lvRoute->addColumn(tr("Type"),100);
    m_lvRoute->addColumn(tr("Status"),100);
	updateList();
}

void CDlgTollCalculator::onBrowserStart()
{
CDlgBrowser2 dlg(&m_network, this);
CNode *pNode = NULL;

	pNode = dlg.goSelectNode ();
	if (pNode) m_cbStart->setEditText (pNode->GetUniqueString());
}

void CDlgTollCalculator::onBrowserDestination()
{
CDlgBrowser2 dlg(&m_network, this);
CNode *pNode = NULL;

	pNode = dlg.goSelectNode ();
	if (pNode) m_cbDestination->setEditText (pNode->GetUniqueString());
}

void CDlgTollCalculator::onSearchStart()
{
	searchStart (false);
}

void CDlgTollCalculator::onSearchDestination()
{
	searchDestination (false);
}

void CDlgTollCalculator::onCalculate()
{
QString strStart = m_cbStart->currentText(), strDest = m_cbDestination->currentText();
QPtrList<CNode> listNode;
CNode *pNodeFrom=NULL, *pNodeTo=NULL, *pNode=NULL;

    m_pbCopy->setEnabled (false);
    m_pbSave->setEnabled (false);

	pNodeFrom = searchStart (true);
	if (pNodeFrom == NULL) return;
	pNodeTo = searchDestination (true);
	if (pNodeTo == NULL) return;

	if (pNodeTo == pNodeFrom)
	{
		QMessageBox::information (this, tr("Information"), tr("Destination and start point are equal."));
		return;
	}

	m_pNodeFrom = pNodeFrom;
	m_pNodeTo = pNodeTo;

	updateList();
	updateCosts();

	if (m_pNodeTo->m_pNearestLink == NULL)
	{
		QMessageBox::critical (this, tr("Error"), tr("Sorry, could not find a way from '%1' to '%2'\nIt's possible that there are some toll free sections between those nodes.").arg (m_pNodeFrom->GetUniqueString()).arg (m_pNodeTo->GetUniqueString()));
		return;
	}
    m_pbCopy->setEnabled (true);
    m_pbSave->setEnabled (true);
}


void CDlgTollCalculator::onOptions()
{
CDlgOptions dlg(this);

	if (dlg.go (&m_config, m_pUpdater, m_pAppTranslator, m_pQtTranslator) == QDialog::Accepted)
	{
		m_config.Save ();
		retranslateStrings();
		reloadDatabase (false);
	}
}


void CDlgTollCalculator::onInformation()
{
CDlgAbout dlg(this);
	dlg.go (&m_network);
}


void CDlgTollCalculator::onCopyClipboard()
{

}


void CDlgTollCalculator::onPollutantChanged( int index )
{
    updateCosts();
}


void CDlgTollCalculator::onAxesChanged( int index )
{
    updateCosts();
}


void CDlgTollCalculator::updateCosts()
{
double dCostsPerKm = GetCostsPerKm (m_cbPollutant->currentItem(), m_cbAxes->currentItem());
double dDistance = 0.0;
	m_tlEURperKm->setText (QString ("%1 EUR").arg (dCostsPerKm, 0, 'f', 2));
	if (m_pNodeTo != NULL && m_pNodeTo->m_pNearestLink != NULL) dDistance = m_pNodeTo->m_fNearestKm;
    
	m_tlDistance->setText(QString ("%1 km").arg(dDistance, 0, 'f', 1));
	m_tlEURTotal->setText(QString ("%1 EUR").arg(dDistance * dCostsPerKm, 0, 'f', 2));
}


void CDlgTollCalculator::updateList()
{
QPtrList<CLink> listLink;
CLink *pLink = NULL;
QListViewItem *pItem = NULL;
CNode *pNode=NULL;
int iOldBAB = 0;
QString str;

	m_pbCopy->setEnabled (false);
	if (m_pNodeFrom == NULL || m_pNodeTo == NULL) return;
    
	m_network.CalculateRoute (m_pNodeFrom, m_pNodeTo, listLink);
	if (m_pNodeTo->m_pNearestLink == NULL) return;

	m_lvRoute->clear ();
	for (pLink = listLink.last(); pLink != NULL; pLink = listLink.prev())
	{
		pNode = pLink->m_pNodeTo;
		if (!m_checkAll->isChecked() && pNode->m_iBAB == iOldBAB) continue;
		if (pLink->m_pNodeFrom->m_iBAB != pLink->m_pNodeTo->m_iBAB) continue;
		if (iOldBAB != 0 && pNode->m_iBAB != iOldBAB)
			str = QString("A %1 / A %2").arg (pNode->m_iBAB).arg(iOldBAB);
		else
			str = QString("A %1").arg (pNode->m_iBAB);
		pItem = new QListViewItem (m_lvRoute, str, pNode->m_strNumber, pNode->m_strName, QString ("%1 km").arg (pNode->m_fNearestKm, 0, 'f', 1), pNode->GetTypeText(), pNode->GetStatusText());
		pItem->setPixmap (0, pNode->GetPixmap16());
		iOldBAB = pNode->m_iBAB;
	}
	pItem = new QListViewItem (m_lvRoute, QString("A %1").arg (m_pNodeFrom->m_iBAB), m_pNodeFrom->m_strNumber, m_pNodeFrom->m_strName, " - ", m_pNodeFrom->GetTypeText(), m_pNodeFrom->GetStatusText());
	pItem->setPixmap (0, m_pNodeFrom->GetPixmap16());
	m_pbCopy->setEnabled (true);
}


void CDlgTollCalculator::onShowAllToggled( bool b )
{
    updateList();
//	m_config.m_bShowAll = b;
//	m_config.Save ();
}


void CDlgTollCalculator::reloadDatabase(const bool bForce)
{
bool bEnable = false;

	m_pNodeFrom = NULL;
	m_pNodeTo = NULL;
	m_lvRoute->clear ();
	updateCosts();

	bEnable = m_network.Load (m_config.m_strDBFile, this, bForce);

	m_cbStart->setEnabled (bEnable);
	m_cbDestination->setEnabled (bEnable);
	m_pbSearchStart->setEnabled (bEnable);
	m_pbSearchDestination->setEnabled (bEnable);
	m_pbBrowserStart->setEnabled (bEnable);
	m_pbBrowserDestination->setEnabled (bEnable);
	m_pbCalculate->setEnabled (bEnable);
	m_pbCopy->setEnabled (false);
}


CNode* CDlgTollCalculator::searchStart( const bool bCalculating )
{
QString str = m_cbStart->currentText();
QPtrList<CNode> listNode;
CNode *pNode=NULL;
QString strAdd = str;

	if (str.isEmpty())
	{
		QMessageBox::information (this, tr("Information"), tr("Please insert a search string for the start point."));
		return NULL;
	}

	m_network.FindNode (str, listNode, m_config.m_bAllowCrossesAsStart);
	if (listNode.isEmpty())
	{
		QMessageBox::information (this, tr("Information"), tr("Sorry, we couldn't find such a start point."));
		return NULL;
	}
	else if (listNode.count() > 1)
	{
		CDlgNodeSelect dlg(this);
		pNode = dlg.go(listNode);
		if (pNode) 
		{
			m_cbStart->setEditText (pNode->GetUniqueString());
			strAdd = pNode->GetUniqueString();
		}
	}
	else
	{
		pNode = listNode.first();
		m_cbStart->setEditText (pNode->GetUniqueString());
		if (!bCalculating) QMessageBox::information (this, tr("Information"), tr("Node was found and is clearly identified."));
	}
	if (bCalculating) 
	{
		m_config.AppendStart (strAdd);
		m_config.FillStartCombo (m_cbStart);
	}
	return pNode;
}


CNode* CDlgTollCalculator::searchDestination( const bool bCalculating )
{
QString str = m_cbDestination->currentText();
QPtrList<CNode> listNode;
CNode *pNode=NULL;
QString strAdd = str;

	if (str.isEmpty())
	{
		QMessageBox::information (this, tr("Information"), tr("Please insert a search string for the destination."));
		return NULL;
	}
	m_network.FindNode (str, listNode, m_config.m_bAllowCrossesAsStart);
	if (listNode.isEmpty())
	{
		QMessageBox::information (this, tr("Information"), tr("Sorry, we couldn't find such a start point."));
		return NULL;
	}
	else if (listNode.count() > 1)
	{
		CDlgNodeSelect dlg(this);
		pNode = dlg.go(listNode);
		if (pNode) 
		{
			m_cbDestination->setEditText (pNode->GetUniqueString());
			strAdd = pNode->GetUniqueString();
		}
	}
	else
	{
		pNode = listNode.first();
		m_cbDestination->setEditText (pNode->GetUniqueString());
		if (!bCalculating) QMessageBox::information (this, tr("Information"), tr("Node was found and is clearly identified."));
	}
	if (bCalculating) 
	{
		m_config.AppendDestination (strAdd);
		m_config.FillDestinationCombo (m_cbDestination);
	}
	return pNode;
}


void CDlgTollCalculator::closeEvent( QCloseEvent *e )
{
	m_config.m_bShowAll = m_checkAll->isChecked();
	m_config.m_iAxes = m_cbAxes->currentItem ();
	m_config.m_iPollutant = m_cbPollutant->currentItem ();
	if (m_pUpdater->m_uLastUpdateCheck)
		m_config.m_uLastUpdate = m_pUpdater->m_uLastUpdateCheck;
	m_config.Save ();
    QDialog::closeEvent (e);
}


void CDlgTollCalculator::onUpdateInfoReceived( void )
{
	disconnect (m_pUpdater, SIGNAL (UpdateInfoReceived()), this, SLOT(onUpdateInfoReceived()));
	if (m_pUpdater->m_iSoftwareVersion > MAUTRECHNER_VERSION)
	{
		QMessageBox::information (this, tr("Update notification"),
			tr ("There exists a newer version of this software.\nPlease download it from http://www.oliver-saal.de/software/mautrechner/"));
	}
	if (m_config.m_iDBVersion < m_pUpdater->m_iDBVersion)
	{
		if (m_config.m_strDBFile.isEmpty() || QMessageBox::question (this, tr("Update notification"), 
			tr("There exists a newer version of the toll tabular.\nWould you download it now?"), 
			QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
		{ // download new database
			m_pUpdater->GetDatabase ();
		}
	}
}


void CDlgTollCalculator::onDatabaseReceived( QString strFile )
{
	m_config.m_strDBFile = strFile;
	m_config.m_iDBVersion = m_pUpdater->m_iDBVersion;
	m_config.Save ();
	reloadDatabase (true);
}


void CDlgTollCalculator::onSave()
{
QString str;
QFile file;

	if (m_pNodeFrom == NULL || m_pNodeTo == NULL) return;

	QString strFile = QFileDialog::getSaveFileName (QString ("%1 - %2.html").arg (m_pNodeFrom->GetUniqueString()).arg(m_pNodeTo->GetUniqueString()), tr("HTML-File (*.html)"), this);
	if (strFile.isEmpty()) return;
	if (strFile.right (4) != "html") strFile += ".html";
	file.setName (strFile);
	file.open (IO_WriteOnly);
	QTextStream ts (&file);
	
	ts << "<html>\n<head></head>\n<body>\n";
	ts << getHTMLText ();
	ts << "</body>\n";

}

QString HTMLString (const QString str)
{
QString strRet = str;

	strRet.replace ("", "&Auml;");
	strRet.replace ("", "&Ouml;");
	strRet.replace ("", "&Uuml;");
	strRet.replace ("", "&auml;");
	strRet.replace ("", "&ouml;");
	strRet.replace ("", "&uuml;");
	strRet.replace ("", "&szlig;");
	strRet.replace (">", "&gt;");
	strRet.replace ("<", "&lt;");
	strRet.replace ("&", "&amp;");
	strRet.replace ("\"", "&quot;");

	return (strRet);
}


QString CDlgTollCalculator::getHTMLText()
{
QString strRet, str1, str2;

	strRet += tr ("<h1>Route</h1>");
	strRet += tr ("<p><i>Generated from TollCalculator V%1 at %2</i></p>").arg (MAUTRECHNER_VERSION_STRING).arg (QDateTime::currentDateTime().toString (Qt::LocalDate));
	str1 = m_labStart->text ();
	str1.remove ('&');
	str2 = m_labDestination->text();
	str2.remove ('&');
	strRet += QString ("<table><tr><td><b>%1</b></td><td>%2</td></tr><tr><td><b>%3</b></td><td>%4</td></tr></table>")
		.arg (str1).arg (m_cbStart->currentText()).arg (str2).arg (m_cbDestination->currentText());

	strRet += ListView2HTML (m_lvRoute);

	strRet += tr ("<h1>Costs</h1>");

	str1 = m_labAxes->text();
	str1.remove ('&');
	strRet += QString ("<table>\n\t<tr><td><b>%1</b></td><td>%2</td></tr>\n").arg (HTMLString (str1)).arg (HTMLString (m_cbAxes->currentText()));

	str1 = m_labPollutant->text();
	str1.remove ('&');
	strRet += QString ("\t<tr><td><b>%1</b></td><td>%2</td></tr>\n").arg (HTMLString (str1)).arg (HTMLString (m_cbPollutant->currentText()));

	str1 = m_labPerKm->text();
	str1.remove ('&');
	strRet += QString ("\t<tr><td><b>%1</b></td><td align='right'>%2</td></tr>\n").arg (HTMLString (str1)).arg (HTMLString (m_tlEURperKm->text()));

	str1 = m_labDistance->text();
	str1.remove ('&');
	strRet += QString ("\t<tr><td><b>%1</b></td><td align='right'>%2</td></tr>\n").arg (HTMLString (str1)).arg (HTMLString (m_tlDistance->text()));

	str1 = m_labTotal->text();
	str1.remove ('&');
	strRet += QString ("\t<tr><td><b>%1</b></td><td align='right'>%2</td></tr>\n").arg (HTMLString (str1)).arg (HTMLString (m_tlEURTotal->text()));

	strRet += "</table>\n";
	return (strRet);
}

